(function() {

    var app = angular.module('FuelIoc');
    var GeoRoService = function($cordovaGeolocation, $ionicPlatform, $http, $timeout, $ionicModal, $cordovaDeviceOrientation) {

        var watchId = null;
        var isFirstTimeLoaded = false;

        //It contains updated current location.
        var currentLocation = null;

        //It contains original location for whitch petrol pumps populated. It will be updated whenever location moves away 500 metres.
        var intialRoPopulatedLocation = null;
        
        var ROS = {};
        var posOptions = {
            enableHighAccuracy: false,
            timeout: 4000,
            maximumAge: 0
        };

        var watchLocation = function(successCallBack, errorCallBack) {
            watchId = $cordovaGeolocation.watchPosition(posOptions);
            watchId.then(
                null,
                function(error) {
                    var errMsg = "";
                    switch (error.code) {
                        case error.PERMISSION_DENIED:
                            errMsg = "User denied the request for Geolocation."
                            break;
                        case error.POSITION_UNAVAILABLE:
                            errMsg = "Location information is unavailable."
                            break;
                        case error.TIMEOUT:
                            errMsg = "The request to get user location timed out."
                            break;
                        case error.UNKNOWN_ERROR:
                            errMsg = "An unknown error occurred."
                            break;
                    }
                    errorCallBack(errMsg);
                },

                function(position) {
                    var lat = position.coords.latitude;
                    var long = position.coords.longitude;
                    currentLocation = new google.maps.LatLng(lat, long);
                    if (!intialRoPopulatedLocation) {
                        intialRoPopulatedLocation = currentLocation;
                        getRos(lat, long, successCallBack, errorCallBack);
                    }
                    else if (calculateDistance(lat, long, intialRoPopulatedLocation.lat(), intialRoPopulatedLocation.lng()) > 0.5) // If location changes more than half kilometre then fress RO will be fetched from DB.
                    {
                        intialRoPopulatedLocation = currentLocation;
                        getRos(lat, long, successCallBack, errorCallBack);
                    }
                    else {
                        successCallBack(null, currentLocation, false);
                    }

                }
            );
        }

        var stopWatchLocation = function() {
            watchId.clearWatch();
        }

        var intialiseGeoRoService = function() {
            $cordovaGeolocation.getCurrentPosition(posOptions).then(function(position) {
                var lat = position.coords.latitude;
                var long = position.coords.longitude;

                currentLocation = new google.maps.LatLng(lat, long);
                intialRoPopulatedLocation = currentLocation;
                getRos(lat, long, successCallBack, errorCallBack);
            },
                function(error) {
                    var errMsg = "";
                    switch (error.code) {
                        case error.PERMISSION_DENIED:
                            errMsg = "User denied the request for Geolocation."
                            break;
                        case error.POSITION_UNAVAILABLE:
                            errMsg = "Location information is unavailable."
                            break;
                        case error.TIMEOUT:
                            errMsg = "The request to get user location timed out."
                            break;
                        case error.UNKNOWN_ERROR:
                            errMsg = "An unknown error occurred."
                            break;
                    }
                    alert(errMsg);
                });
        }

        var getRos = function(lat, long, success, error) {
            $http.get('https://xsparsh.indianoil.in:8443/soa-infra/resources/default/RDBFuel/RSRoInfo/',
                {
                    params: {
                        longitude: long,
                        latitude: lat
                    }
                }).success(function(data, status) {
                    if (data && data.VwMobRoinfo && data.VwMobRoinfo.length > 0) {
                        var marker;
                        var location = new google.maps.LatLng(lat, long);
                        data.VwMobRoinfo = calculateDistances(data.VwMobRoinfo, lat, long);
                        data.VwMobRoinfo = sortRosBasedOnDistance(data.VwMobRoinfo, 'distance');

                        for (i = 0; i < data.VwMobRoinfo.length; i++) {

                            marker = new google.maps.Marker(
                                {
                                    position: new google.maps.LatLng(data.VwMobRoinfo[i].latitude, data.VwMobRoinfo[i].longitude),
                                    map: null,
                                    icon: 'img/Map/fuelIcon.png',
                                    animation: google.maps.Animation.DROP
                                });
                            data.VwMobRoinfo[i].marker = marker;
                            data.VwMobRoinfo[i].products = formatRoProducts(data.VwMobRoinfo[i]);
                            data.VwMobRoinfo[i].services = formatRoServices(data.VwMobRoinfo[i].roServices);
                            data.VwMobRoinfo[i].IsFilterSatisfied = true;
                            if (i == 0) {
                                data.VwMobRoinfo[i].isNearest = true;
                                if(data.VwMobRoinfo[i].promo == 'Y')
                                {
                                    data.VwMobRoinfo[i].marker.setIcon('img/Map/fuelIconNO.png');
                                }
                                else
                                {
                                    data.VwMobRoinfo[i].marker.setIcon('img/Map/fuelIconN.png');
                                }
                                
                            }
                            else {
                                if(data.VwMobRoinfo[i].promo == 'Y')
                                {
                                    data.VwMobRoinfo[i].marker.setIcon('img/Map/fuelIconO.png');
                                }
                                data.VwMobRoinfo[i].isNearest = false;
                            }
                        }
                        if (success) {
                            success(data.VwMobRoinfo, location, true);
                        }
                        else {
                            successCallBack(data.VwMobRoinfo, location, true);
                        }
                    }
                    else {
                        if (success) {
                            success(null, location, true);
                        }
                        else {
                            successCallBack(null, location, true);
                        }

                    }
                }).error(onError);

            function onError(data) {
                if (error) {
                    error('RO service not available');
                }
                else {
                    errorCallBack('RO service not available');
                }
            }
        }

        var returnRos = function() {
            return ROS;
        }
        var returnCL = function() {
            return currentLocation;
        }

        //Supporting functions
        function calculateDistances(rots, lat, long) {
            for (i = 0; i < rots.length; i++) {
                rots[i].distance = calculateDistance(rots[i].latitude, rots[i].longitude, lat, long);
            }
            return rots;
        }

        function sortRosBasedOnDistance(rots, key) {
            return rots.sort(function(a, b) {
                var x = parseFloat(a[key]);
                var y = parseFloat(b[key]);
                return ((x < y) ? -1 : ((x > y) ? 1 : 0));
            });
        }

        function calculateDistance(lat1, lon1, lat2, lon2) {
            var unit = 'K';
            var radlat1 = Math.PI * lat1 / 180;
            var radlat2 = Math.PI * lat2 / 180;
            var theta = lon1 - lon2;
            var radtheta = Math.PI * theta / 180;
            var dist = Math.sin(radlat1) * Math.sin(radlat2) + Math.cos(radlat1) * Math.cos(radlat2) * Math.cos(radtheta);
            dist = Math.acos(dist);
            dist = dist * 180 / Math.PI;
            dist = dist * 60 * 1.1515;
            if (unit == "K") {
                dist = dist * 1.609344;
            }
            if (unit == "N") {
                dist = dist * 0.8684;
            }
            return dist;
        }

        function formatRoProducts(ro) {
            var products = [];
            if (JSON.stringify(ro.ms).indexOf('@nil') == -1) {
                products.push({ name: "PETROL", cost: ro.ms, IsAvailable: true });
            }
            else {
                products.push({ name: "PETROL", cost: ro.ms, IsAvailable: false });
            }
            if (JSON.stringify(ro.hsd).indexOf('@nil') == -1) {
                products.push({ name: "DIESEL", cost: ro.hsd, IsAvailable: true });
            }
            else {
                products.push({ name: "DIESEL", cost: ro.hsd, IsAvailable: false });
            }

            if (JSON.stringify(ro.xp).indexOf('@nil') == -1) {
                products.push({ name: "XTRAPREMIUM", cost: ro.xp, IsAvailable: true });
            }
            else {
                products.push({ name: "XTRAPREMIUM", cost: ro.xp, IsAvailable: false });
            }

            if (JSON.stringify(ro.xm).indexOf('@nil') == -1) {
                products.push({ name: "XTRAMILE", cost: ro.xm, IsAvailable: true });
            }
            else {
                products.push({ name: "XTRAMILE", cost: ro.xm, IsAvailable: false });
            }
            return products;

        }

        function formatRoServices(roServices) {
            var services = roServices.split(',');
            return services;
        }
        function successCallBack(data, cloaction, val) {
            isFirstTimeLoaded = true;
            ROS = data;
        }
        function errorCallBack(error) {
            isFirstTimeLoaded = true;
            //alert(error);
        }
        
        function isDataLoaded()
        {
            return isFirstTimeLoaded;
        }

        return {
            getRos: getRos,
            watchLocation: watchLocation,
            stopWatchLocation: stopWatchLocation,
            intialiseGeoRoService: intialiseGeoRoService,
            returnRos: returnRos,
            returnCL: returnCL,
            isDataLoaded : isDataLoaded
        }

    };
    app.factory("GeoRoService", GeoRoService);

} ());